import numpy as np

from . import nodes


def number_of_cells(shape):
    """Number of cells is a structured quad grid.

    Parameters
    ----------
    shape : tuple of int
        Shape of grid of nodes.

    Returns
    -------
    int :
        Number of cells in the grid.

    Examples
    --------
    >>> from landlab.grid.structured_quad.cells import number_of_cells
    >>> number_of_cells((3, 4))
    2
    """
    return np.prod(np.array(shape) - 2)


def shape_of_cells(shape):
    """Shape of cells in a structured quad grid.

    Parameters
    ----------
    shape : tuple of int
        Shape of grid of nodes.

    Returns
    -------
    int :
        Shape of cells in grid.

    Examples
    --------
    >>> from landlab.grid.structured_quad.cells import shape_of_cells
    >>> shape_of_cells((3, 4))
    (1, 2)
    """
    return tuple(np.array(shape) - 2)


def node_id_at_cells(shape):
    """Node ID at each cell.

    Parameters
    ----------
    shape : tuple of int
        Shape of grid of nodes.

    Returns
    -------
    ndarray :
        ID of node associated with each cell.

    Examples
    --------
    >>> from landlab.grid.structured_quad.cells import node_id_at_cells
    >>> node_id_at_cells((3, 4))
    array([[5, 6]])
    """
    node_ids = nodes.node_ids(shape)
    return node_ids[1:-1, 1:-1].copy().reshape(shape_of_cells(shape))


def cell_ids(shape):
    """IDs of all cells.

    Parameters
    ----------
    shape : tuple of int
        Shape of grid of nodes.

    Returns
    -------
    ndarray :
        ID of node associated with each cell.

    Examples
    --------
    >>> from landlab.grid.structured_quad.cells import cell_ids
    >>> cell_ids((3, 4))
    array([[0, 1]])
    """
    return np.arange(number_of_cells(shape)).reshape(shape_of_cells(shape))


class StructuredQuadCellGrid(object):
    def __init__(self, shape):
        self._shape = shape_of_cells(shape)
        self._number_of_cells = np.prod(self._shape)
        self._node_at_cell = node_id_at_cells(shape)

    @property
    def number_of_cells(self):
        return self._number_of_cells

    @property
    def shape(self):
        return self._shape

    @property
    def number_of_cell_rows(self):
        return self.shape[0]

    @property
    def number_of_cell_columns(self):
        return self.shape[1]

    @property
    def node_at_cell(self):
        return self._node_at_cell

    def number_of_vertices_at_cell(self, cell):
        return 4



#! /bin/env python

from landlab.io.vtk.writer import VtkWriter
from landlab.io.vtk.vtktypes import VtkUniformRectilinear
from landlab.io.vtk.vtkxml import (VtkRootElement, VtkGridElement,
                                   VtkPieceElement, VtkCoordinatesElement,
                                   VtkPointDataElement, VtkCellDataElement,
                                   VtkExtent)


class VtkUniformRectilinearWriter(VtkWriter):
    _vtk_grid_type = VtkUniformRectilinear

    def construct_field_elements(self, field):
        extent = VtkExtent(field.shape[::-1])
        origin = VtkOrigin(field.origin[::-1], field.spacing[::-1])
        spacing = VtkSpacing(field.spacing[::-1])

        element = {
            'VTKFile':
                VtkRootElement(VtkUniformRectilinear),
            'Grid':
                VtkGridElement(VtkUniformRectilinear, WholeExtent=extent,
                               Origin=origin, Spacing=spacing),
            'Piece':
                VtkPieceElement(Extent=extent),
            'PointData':
                VtkPointDataElement(field.at_node, append=self.data,
                                    encoding=self.encoding),
            'CellData':
                VtkCellDataElement(field.at_cell, append=data,
                                   encoding=encoding),
        }

        return element



#! /usr/env/python
"""

2D numerical model of shallow-water flow over topography read from a DEM, using
the Bates et al. (2010) algorithm for storage-cell inundation modeling.

1st stab at componentizing this routine, following GT, by DEJH, Oct 2013.

"""

from landlab import ModelParameterDictionary
import numpy as np

class SurfaceFlowTransport(object):
    
    def __init__(self, grid, input_stream):
        #grid here is a true model field. i.e., we should be able to do grid.at_node['topographic_elevation']
        #input_stream is a text file, entered in format './my_file.txt'
        self.grid = grid
        
        inputs = ModelParameterDictionary(input_stream)
        self.n = inputs.read_float('n')              # roughness coefficient (Manning's n)
        self.g = inputs.read_float('g')               # gravitational acceleration (m/s2)
        self.alpha = inputs.read_float('alpha')       # time-step factor (ND; from Bates et al., 2010)
        self.tau_crit = inputs.read_float('tau_crit') # critical shear stress, pascals
        self.mpm = inputs.read_float('mpm')          # sed trans coefficient
        self.erode_start_time = inputs.read_float('erode_start_time') # allows an offset between when flow starts and when we start to allow erosion
        
        
        #test the necessary fields are all already present:
        try:
            self.z = grid.at_node['topographic_elevation']
        except:
            print 'elevations not found in grid!'
        try:
            self.h = grid.at_node['planet_surface__water_depth']
        except:
            print 'initial water depths not found in grid!'
        
        #build the internally necessary params:
        self.rhog = 9810.          # water unit weight, kg/m2s2 (N/m3)
        self.q = grid.create_active_link_array_zeros()       # unit discharge (m2/s)
        self.dhdt = grid.create_node_array_zeros()           # rate of water-depth change
        self.tau = grid.create_active_link_array_zeros()     # shear stress (Pa)
        self.qs = grid.create_active_link_array_zeros()      # sediment flux (m2/s)
        self.dqsds = grid.create_node_array_zeros()
        self.dzdt = self.dhdt
        self.dzaccum = grid.create_node_array_zeros()
        self.zm = grid.create_node_array_zeros()
        self.zm[:] = self.z[:]
        
    def set_and_return_dynamic_timestep(self):
        # Calculate time-step size for this iteration (Bates et al., eq 14)
        self.dtmax = self.alpha*self.grid.dx/np.sqrt(self.g*np.amax(self.h))
        return self.dtmax
        
    def set_timestep(self, timestep_in):
        """
        This fn allows overriding of the inbuilt dynamic timestepping, if, e.g.,
        another component requires a shorter timestep.
        Function assumes you have already called self.
        set_and_return_dynamic_timestep() to establish what the min should be.
        """
        if timestep_in <= self.dtmax:
            self.dtmax = timestep_in
        else:
            raise RuntimeError('Attempting to manually set an unstable timestep! Abort! Abort!')
        
    def transport_sed(self, elapsed_time):
        #load data for speed and clarity:
        grid = self.grid
        z = self.z
        h = self.h
        g = self.g
        n = self.n
        tau_crit = self.tau_crit
        q = self.q
        qs = self.qs
        tau = self.tau
        rhog = self.rhog
        alpha = self.alpha
        mpm = self.mpm
        zm = self.zm
        dtmax = self.dtmax
        erode_start_time = self.erode_start_time
        ten_thirds = 10./3.
        
        interior_cells = grid.get_active_cell_node_ids()
        

        # Calculate the effective flow depth at active links. Bates et al. 2010
        # recommend using the difference between the highest water-surface
        # and the highest bed elevation between each pair of cells.
        zmax = grid.max_of_link_end_node_values(z)
        w = h+z   # water-surface height
        wmax = grid.max_of_link_end_node_values(w)
        hflow = wmax - zmax
            
        # Calculate water-surface slopes
        water_surface_slope = grid.calculate_gradients_at_active_links(w)
        
        # Calculate the unit discharges (Bates et al., eq 11)
        q = (q-g*hflow*dtmax*water_surface_slope)/ \
            (1.+g*hflow*dtmax*n*n*abs(q)/(hflow**ten_thirds))

        # Calculate shear stress and sediment flux
        tau = -rhog*hflow*water_surface_slope
        tauex = abs(tau)-tau_crit
        tauex[np.where(tauex<0.)] = 0.
        qs = np.sign(tau)*mpm*pow(tauex,1.5)

        # Calculate water-flux divergence at nodes
        dqds = grid.calculate_flux_divergence_at_nodes(q)
        dqsds = grid.calculate_flux_divergence_at_nodes(qs)
        
        # Calculate rate of change of water depth
        dhdt = -dqds
        dzdt = -dqsds
            
        # Second time-step limiter (experimental): make sure you don't allow
        # water-depth to go negative
        excess_time = 0.
        if np.amin(dhdt) < 0.:
            shallowing_locations = np.where(dhdt<0.)
            time_to_drain = -h[shallowing_locations]/dhdt[shallowing_locations]
            dtmax2 = alpha*np.amin(time_to_drain)
            min_timestep_ratio = int(dtmax//dtmax2)
            if min_timestep_ratio: #any value other than 0, i.e., dtmax2<dtmax
                excess_time = dtmax/dtmax2 - min_timestep_ratio
                dt = dtmax2
            else:
                dt =  dtmax
        else:
            min_timestep_ratio = 0
            dt = dtmax

        #perform a loop if we had to subdivide the tstep above:
        for i in xrange(min_timestep_ratio+1):
            # Update the water-depth field
            h[interior_cells] += dhdt[interior_cells]*dt
            if elapsed_time >= erode_start_time:
                #z[interior_cells] = z[interior_cells] + dzdt[interior_cells]*dt
                zm[interior_cells] += dzdt[interior_cells]*dt
                #dzaccum[interior_cells] += dzdt[interior_cells]*dt
        if excess_time:
            h[interior_cells] += dhdt[interior_cells]*dt*excess_time
            if elapsed_time >= erode_start_time:
                zm[interior_cells] += dzdt[interior_cells]*dt*excess_time
            
        #Now we need to embed the results back into the object, and the grid
        self.z = z
        self.h = h
        self.q = q
        self.qs = qs
        self.tau = tau
        self.zm = zm
        self.grid['node']['topographic_elevation'] = zm
        self.grid['node']['planet_surface__water_depth'] = h
        
        return self.grid
        



#! /usr/env/python

#-----------------------------------------------------------------------
#
# diffusion2Dtest.py: Tests ModelGrid() class by implementing a 2D
# diffusion code.
#
# GT, July 2010
#-----------------------------------------------------------------------

import numpy as np

from landlab import RasterModelGrid

import pylab
from pylab import plot, draw, show, contour


def main():
    # Initialize

    # User-defined parameters
    nr = 130     # number of rows
    nc = 3     # number of columns
    dx = 0.25   # cell spacing
    s = 0.001  # source
    k = 0.01   # flux coefficient
    dt = 0.25*dx**2.0/k   # time step
    print 'dt=', dt
    nt = 40000    # number of iterations

    # Setup
    mg = RasterModelGrid(nr, nc, dx)
    u = mg.zeros(centering='cell')
    interior_cells = mg.get_interior_cells()
    dudt = mg.zeros(centering='cell')

    # Boundaries
    mg.set_noflux_boundaries(False, True, False, True)

    # Process
    for i in range(0, nt):
        print i
        g = mg.calculate_face_gradients(u)
        q = -k*g
        dqds = mg.calculate_flux_divergences(q)
        for c in interior_cells:
            dudt[c] = s - dqds[c]
        u = u + dudt * dt
        u = mg.update_noflux_boundaries(u)

    # Finalize
    ur = mg.cell_vector_to_raster(u)
    print 'Max u = ', max(u)
    contour(ur)
    show()


if __name__ == '__main__':
    main()



#! /usr/env/python

"""
route_flow_dn.py:
    
Given a ModelGrid, calculates single-path (steepest direction) flow directions,
drainage area, and (optionally) discharge.

The "dn" in the name means that this is a generalization of the D8 algorithm,
for a grid in which a node has N neighbors (N might happen to be 8, or not).

Created GT Nov 2013
Modified to save data to grid directly, DEJH March 2014
"""

import landlab
#from landlab import RasterModelGrid
from landlab.components.flow_routing import flow_direction_DN
#reload(flow_direction_DN)
from landlab.components.flow_accum import flow_accum_bw
import numpy
from scipy import weave
#from scipy.weave.build_tools import CompileError

#output_suppression_flag = True

class FlowRouter():
    """
    This class implements single-path (steepest direction) flow routing, and 
    calculates flow directions, drainage area, and (optionally) discharge. 
    
    It initializes with a reference to a ModelGrid of any kind. Optionally, it
    can also take *value_field*, the string which is the name of the elvation 
    field in the model grid to use to route flow. If *value_field* is not
    provided, it defaults to 'topographic_elevation'.
    
    The primary method of this class is :func:`route_flow`.
    """
    
    def __init__(self, model_grid, value_field='topographic_elevation'):
        
        # We keep a local reference to the grid
        self._grid = model_grid
        self.value_field = value_field
        
        # We'll also keep track of the active links; if raster, then these are
        # the "D8" links; otherwise, it's just activelinks
        if type(model_grid) is landlab.grid.raster.RasterModelGrid:
            dal, d8f, d8t = model_grid.d8_active_links()
            self._active_links = dal
            self._activelink_from = d8f
            self._activelink_to = d8t
        else:
            self._active_links = model_grid.active_links
            self._activelink_from = model_grid.activelink_fromnode
            self._activelink_to = model_grid.activelink_tonode
            
        # Keep track of the following variables:
        #   - drainage area at each node
        #   - receiver of each node
        self.drainage_area = model_grid.add_zeros('node', 'drainage_area')
        self.receiver = model_grid.create_node_array_zeros('flow_receiver')
        self.steepest_slope = model_grid.create_node_array_zeros('steepest_slope')
        self.discharges = model_grid.create_node_array_zeros('water_discharges')
        self.upstream_ordered_nodes = model_grid.create_node_array_zeros('upstream_ID_order')
        self.links_to_receiver = model_grid.create_node_array_zeros('links_to_flow_receiver')
        
        self.weave_flag = model_grid.weave_flag
        
        
    def route_flow(self, elevs=None, grid=None, runoff_rate=1.0,
                   boundary_nodes=None):
        """
        Routes surface-water flow by (1) assigning to each node a single 
        drainage direction, and then (2) adding up the number of nodes that
        contribute flow to each node on the grid (including the node itself).
        If a scalar is specified for cell_area, computes the total surface 
        contributing area by assuming that each cell has the same surface area.
        If an array is given (with length equal to the number of nodes), these
        areas are used for each cell. Likewise, runoff_rate, in length per
        time (volume per area per time) may be given either as a scalar
        (identical for each cell) or as an array whose length is the number of
        nodes in the grid.
        
        Takes:
            - Either *elevs*, an array of node elevations, or *grid*, a 
              reference to a ModelGrid.
              
        Takes as optional inputs:
            - *runoff_rate*, a float (for constant rainfall) or array (for
              spatially variable rainfall) of runoff rates, such that drainage
              area is in volume, rather than number of upstream cells.
             
            - Note that this module **no longer** takes *node_cell_area* as an
              optional input. Node cell area will always be read from the
              grid supplied.
        
        Stores as ModelGrid fields, or returns, if *elevs* was provided rather
        than *grid*:
            - Node array of receivers (nodes that receive flow): 
              *'flow_receiver'*
            - Node array of drainage areas: *'drainage_area'*
            - Node array of discharges: *'water_discharges'*
            - Node array of steepest downhill slopes: *'steepest_slope'*
            - Node array containing downstream-to-upstream ordered list of node
              IDs: *'upstream_ID_order'*
            - Node array containing ID of link that leads from each node to its
              receiver (or ITS OWN ID if there is no receiver):
              *'links_to_flow_receiver'*
        
        Returns, if *grid* was provided:
            - the modified grid object
        
        Examples
        --------
        >>> import numpy as np
        >>> from landlab import RasterModelGrid
        >>> from landlab.components.flow_routing.route_flow_dn import FlowRouter
        >>> mg = RasterModelGrid(5, 4, 1.0)
        >>> elev = np.array([0.,  0.,  0., 0.,
        ...                  0., 21., 10., 0.,
        ...                  0., 31., 20., 0.,
        ...                  0., 32., 30., 0.,
        ...                  0.,  0.,  0., 0.])
        >>> mg.set_closed_boundaries_at_grid_edges(False, True, True, True)
        >>> fr = FlowRouter(mg)
        >>> r, a, q, ss, s, rl = fr.route_flow(elevs=elev)
        >>> r
        array([ 0,  1,  2,  3,  4,  1,  2,  7,  8,  6,  6, 11, 12, 10, 10, 15, 16,
               17, 18, 19])
        >>> a
        array([ 1.,  2.,  6.,  1.,  1.,  1.,  5.,  1.,  1.,  1.,  3.,  1.,  1.,
                1.,  1.,  1.,  1.,  1.,  1.,  1.])

        Now let's change the cell area and the runoff rates:
        
        >>> mg = RasterModelGrid(5, 4, 10.) #so cell area==100.
        >>> mg.set_closed_boundaries_at_grid_edges(False, True, True, True)
        >>> fr = FlowRouter(mg)
        >>> runoff_rate = np.arange(mg.number_of_nodes)
        >>> r, a, q, ss, s, rl = fr.route_flow(elevs=elev, runoff_rate=runoff_rate)
        >>> q
        array([    0.,   600.,  5400.,   300.,   400.,   500.,  5200.,   700.,
                 800.,   900.,  3700.,  1100.,  1200.,  1300.,  1400.,  1500.,
                1600.,  1700.,  1800.,  1900.])
        
        """
        
        #if elevs is not provided, default to stored grid values, which must be provided as grid
        if elevs is None:
            if grid is not None:
                self._grid = grid
                elevs = grid['node'][self.value_field]
            else:
                raise ValueError('Either an elevation array or a copy of the grid must be provided!')
        
        node_cell_area = self._grid.forced_cell_areas
            
        
        # Calculate the downhill-positive slopes at the d8 active links
        #TODO: generalize to use EITHER D8, if raster, or just active links,
        # otherwise.
        link_slope = -self._grid.calculate_gradients_at_d8_active_links(elevs)
        # Find the baselevel nodes
        (baselevel_nodes, ) = numpy.where(numpy.logical_or(self._grid.node_status==1, self._grid.node_status==2))

        # Calculate flow directions
        receiver, steepest_slope, sink, recvr_link  = \
            flow_direction_DN.flow_directions(elevs, self._active_links, 
                                         self._activelink_from,
                                         self._activelink_to, link_slope, 
                                         grid=grid,
                                         baselevel_nodes=baselevel_nodes, 
                                         use_weave=self.weave_flag)
#############grid=None???
        
        # TODO: either need a way to calculate and return the *length* of the
        # flow links, OR the caller has to handle the raster / non-raster case.
        
        #print 'sinks:', sink

        # Calculate drainage area, discharge, and ...
        a, q, s = flow_accum_bw.flow_accumulation(receiver, sink,
                                                  node_cell_area, runoff_rate,
                                                  boundary_nodes, self.weave_flag)
                                                  
        #added DEJH March 2014:
        #store the generated data in the grid
        self._grid['node']['drainage_area'] = a
        self._grid['node']['flow_receiver'] = receiver
        self._grid['node']['steepest_slope'] = steepest_slope
        self._grid['node']['water_discharges'] = q
        self._grid['node']['upstream_ID_order'] = s
        self._grid['node']['links_to_flow_receiver'] = recvr_link
        self._grid['node']['flow_sinks'] = numpy.zeros_like(receiver, dtype=bool)
        self._grid['node']['flow_sinks'][sink] = True
        

        if grid:
            return self._grid
        else:
            return receiver, a, q, steepest_slope, s, recvr_link

    @property
    def node_drainage_area(self):
        return self._grid['node']['drainage_area']

    @property
    def node_receiving_flow(self):
        return self._grid['node']['flow_receiver']

    @property
    def node_steepest_slope(self):
        return self._grid['node']['steepest_slope']

    @property
    def node_water_discharge(self):
        return self._grid['node']['water_discharges']

    @property
    def node_order_upstream(self):
        return self._grid['node']['upstream_ID_order']

    @property
    def link_to_flow_receiving_node(self):
        return self._grid['node']['links_to_flow_receiver']


if __name__ == '__main__':
    import doctest
    doctest.testmod()



#################################################################
##
##  'Field' concept is implemented for Single Species Vegetation.
##
##  Sai Nudurupati and Erkan Istanbulluoglu - 04Mar2014
#################################################################

from landlab import Component

import numpy as np

_VALID_METHODS = set(['Grid'])

def assert_method_is_valid(method):
    if method not in _VALID_METHODS:
        raise ValueError('%s: Invalid method name' % method)


class Vegetation( Component ):
    """
    Landlab component that implements 1D and 2D vegetation dynamics
    model.

    >>> from landlab import RasterModelGrid
    >>> from landlab.components.single_vegetation.single_vegetation_field import Vegetation
    >>> grid = RasterModelGrid(5, 4, 1.e4)
    >>> veg = Vegetation(grid)
    >>> veg.name
    'Vegetation'
    """
    _name = 'Vegetation'

    _input_var_names = set([
        'ActualEvapotranspiration',
        'WaterStress',
        'PotentialEvapotranspiration',
    ])

    _output_var_names = set([
        'LiveLeafAreaIndex',
        'DeadLeafAreaIndex',
        'VegetationCover',
        'LiveBiomass',
        'DeadBiomass',
    ])

    _var_units = {
        'LiveLeafAreaIndex' : 'None',
        'DeadLeafAreaIndex' : 'None',
        'VegetationCover'  : 'None',
        'ActualEvapotranspiration': 'mm',
        'PotentialEvapotranspiration': 'mm',
        'WaterStress': 'Pa',
        'LiveBiomass' : 'g DM m^-2 d^-1',
        'DeadBiomass' : 'g DM m^-2 d^-1',
    }

    def __init__(self, grid, **kwds):
        self._method = kwds.pop('method', 'Grid')
        self._WUE = kwds.pop('WUE', 0.01)
        self._LAI_max = kwds.pop('LAI_MAX', 2.)
        self._cb = kwds.pop('CB', 0.0047)
        self._cd = kwds.pop('CD', 0.009)
        self._ksg = kwds.pop('KSG', 0.012)
        self._kdd = kwds.pop('KDD', 0.013)
        self._kws = kwds.pop('KWS', 0.02)
        self._Blive_init = kwds.pop('BLIVE_INI', 102.)
        self._Bdead_init = kwds.pop('BDEAD_INI', 450.)
        self._ETthresholdup = kwds.pop('ETTup', 3.8)
        self._ETthresholddown = kwds.pop('ETTdwn', 6.5)
        self._ETdmax = kwds.pop('ETdmax', 10)

        assert_method_is_valid(self._method)

        super(Vegetation, self).__init__(grid, **kwds)

        for name in self._input_var_names:
            if name not in self.grid.at_cell:
                self.grid.add_zeros('cell', name, units=self._var_units[name])

        for name in self._output_var_names:
            if name not in self.grid.at_cell:
                self.grid.add_zeros('cell', name, units=self._var_units[name])

        self._cell_values = self.grid['cell']

        self._Blive_ini = self._Blive_init * np.ones(self.grid.number_of_cells)
        self._Bdead_ini = self._Bdead_init * np.ones(self.grid.number_of_cells)

    def update(self, **kwds):

        PETthreshold_ = kwds.pop('PotentialEvapotranspirationThreshold', 0)
        Tb = kwds.pop('Tb', 24.)
        Tr = kwds.pop('Tr', 0.01)
        PET = self._cell_values['PotentialEvapotranspiration']
        ActualET = self._cell_values['ActualEvapotranspiration']
        Water_stress = self._cell_values['WaterStress']

        self._LAIlive = self._cell_values['LiveLeafAreaIndex']
        self._LAIdead = self._cell_values['DeadLeafAreaIndex']
        self._Blive = self._cell_values['LiveBiomass']
        self._Bdead = self._cell_values['DeadBiomass']
        self._VegCov = self._cell_values['VegetationCover']

        if PETthreshold_ == 1:
            PETthreshold = self._ETthresholdup
        else:
            PETthreshold = self._ETthresholddown

        for cell in range(0, self.grid.number_of_cells):

            LAIlive =  min (self._cb * self._Blive_ini[cell], self._LAI_max)
            LAIdead =  min (self._cd * self._Bdead_ini[cell], (self._LAI_max   \
                            - LAIlive))

            if PET[cell] > PETthreshold:  # Growing Season

                NPP = max((ActualET[cell]/(Tb+Tr))*                            \
                            self._WUE*24.*0.55*1000., 0.001)
                Bmax = (self._LAI_max - LAIdead)/self._cb
                Yconst = (1./((1./Bmax)+(((self._kws*Water_stress[cell])+      \
                            self._ksg)/NPP)))
                Blive = (self._Blive_ini[cell] - Yconst) *np.exp(-(NPP/Yconst)*\
                            ((Tb+Tr)/24.)) + Yconst
                Bdead = (self._Bdead_ini[cell] + (Blive - max(Blive *          \
                            np.exp(-self._ksg * Tb/24),0.00001)))*             \
                            np.exp(-self._kdd * min( PET[cell]/10., 1. ) *     \
                            Tb/24.)

            else:                                 # Senescense

                Blive = max(self._Blive_ini[cell] * np.exp((-2) * self._ksg *  \
                            Tb/24.), 1. )
                Bdead = max( (self._Bdead_ini[cell] + ( self._Blive_ini[cell]  \
                            -max( self._Blive_ini[cell]*np.exp( (-2) *         \
                            self._ksg * Tb/24.), 0.000001)))*                  \
                            np.exp( (-1)*self._kdd *                           \
                            min( PET[cell]/10., 1. ) * Tb/24.), 0. )

            LAIlive =  min( self._cb * Blive, self._LAI_max )
            LAIdead =  min( self._cd * Bdead, ( self._LAI_max - LAIlive ) )
            Vt = 1. - np.exp(-0.75 * (LAIlive + LAIdead))

            self._LAIlive[cell] = LAIlive
            self._LAIdead[cell] = LAIdead
            self._VegCov[cell] = Vt
            self._Blive[cell] = Blive
            self._Bdead[cell] = Bdead

        self._Blive_ini = self._Blive
        self._Bdead_ini = self._Bdead



#! /usr/env/python
# -*- coding: utf-8 -*-
"""
Component for detachment-limited fluvial incision using a simple power-law model.

I = K Q^m S^n

I=incision rate (M Y^(-1) )
K=bedrock erodibility (M^(1-3m) Y^(m-1) ) #read in from input file
Q= fluvial discharge (M^3 Y^-1 )
S=slope of landscape (negative of the gradient in topography, dimensionless, 
and only applies on positive slopes)
m, n =exponents, read in from input file

NOTE, in units above M=meters.  This component assumes that variables are given
in units of meters and years, including rainfall!

NOTE, Only incision happens in this class.  NO DEPOSITION.  
NO TRACKING OF SEDIMENT FLUX.

This assumes that a grid has already been instantiated.

To run this, first instantiate the class member, then run one storm
    incisor = PowerLawIncision('input_file_name',grid)
    z = incisior.run_one_storm(grid, z, rainrate=optional, storm_duration=optional)
    
Note that the component knows the default rainfall rate (m/yr) and storm duration (yr)
so these values need not be passed in.  Elevationare eroded and sent back.
    
 
"""

#import landlab
from landlab import ModelParameterDictionary
from landlab.components.flow_routing.flow_routing_D8 import RouteFlowD8
from landlab.components.flow_accum.flow_accumulation2 import AccumFlow
import numpy as np
#import matplotlib.pyplot as plt

class PowerLawIncision(object):
    
    def __init__(self, input_stream, grid, current_time=0.):
        
        self.grid = grid
        #create and initial grid if one doesn't already exist
        #if self.grid==None:
        #    self.grid = create_and_initialize_grid(input_stream)
        
        self.current_time = current_time
        self.initialize(grid, input_stream)
        
    def initialize(self, grid, input_stream):
        
        # Create a ModelParameterDictionary for the inputs
        if type(input_stream)==ModelParameterDictionary:
            inputs = input_stream
        else:
            inputs = ModelParameterDictionary(input_stream)
        
        # Read input/configuration parameters
        self.m = inputs.get('M_EXPONENT', ptype=float)
        self.n = inputs.get('N_EXPONENT', ptype=float)
        self.K = inputs.get('K_COEFFICIENT', ptype=float)
        self.rainfall_myr = inputs.get('RAINFALL_RATE_M_PER_YEAR', ptype=float)
        self.rain_duration_yr = inputs.get('RAIN_DURATION_YEARS', ptype=float)
        self.frac = 0.9 #for time step calculations
        
        #print "Rainfall duration ", self.rain_duration
        
        # Set up state variables
        self.I = grid.zeros(centering='node')    # incision rate (M/Y)
        
    def run_one_storm(self, grid, z, rainrate=None, storm_dur=None):
        
        if rainrate==None:
            rainrate = self.rainfall_myr
        if storm_dur==None:
            storm_dur = self.rain_duration_yr
            
        m=self.m
        n=self.n
        K=self.K
        frac = self.frac
        
        #interior_nodes are the nodes on which you will be calculating incision 
        interior_nodes = grid.get_active_cell_node_ids()
        
        #instantiate variable of type RouteFlowD8 Class
        flow_router = RouteFlowD8(len(z))
        #initial flow direction
        flowdirs, max_slopes = flow_router.calc_flowdirs(grid,z)
        #print "elevations in runonestorm ",grid.node_vector_to_raster(z)
        #insantiate variable of type AccumFlow Class
        accumulator = AccumFlow(grid)
        #initial flow accumulation
        drain_area = accumulator.calc_flowacc(grid, z, flowdirs)
        
        time=0
        dt = storm_dur
        while time < storm_dur:
            #Calculate incision rate, should be in m/yr, should be negative
            #First make sure that there are no negative (uphill slopes)
            #Set those to zero, because incision rate should be zero there.
            max_slopes.clip(0)
            I=-K*np.power(rainrate*drain_area,m)*np.power(max_slopes,n)
            
            #print "incision rates ",grid.node_vector_to_raster(I)
            #print "flow dirs ",grid.node_vector_to_raster(flowdirs)
            #print "max slopes ",grid.node_vector_to_raster(max_slopes)
            #print "drainage area ", grid.node_vector_to_raster(drain_area)
            
            #Do a time-step check
            #If the downstream node is eroding at a slower rate than the 
            #upstream node, there is a possibility of flow direction reversal,
            #or at least a flattening of the landscape. 
            #Limit dt so that this flattening or reversal doesn't happen.
            #How close you allow these two points to get to eachother is
            #determined by the variable frac.
            for i in interior_nodes:
                dzdtdif = I[flowdirs[i]]-I[i]
                if dzdtdif > 0:
                    dtmin = frac*(z[i]-z[flowdirs[i]])/dzdtdif
                    #Nic do you need a test here to make sure that dtmin 
                    #isn't too small?  Trying that out
                    if dtmin < dt:
                        if dtmin>0.001*dt:
                            dt = dtmin
                        else:
                            dt = 0.001*dt
            
            #should now have a stable timestep.
            #reduce elevations.
            z=I*dt+z
            
            #update elapsed time
            time=dt+time
            
            #check to see that you are within 0.01% of time
            #if so, done
            #otherwise, reset everything for next loop
            
            if time > 0.9999*storm_dur:
                #done!
                time = storm_dur
            else:
                #not done, reset everything
                #update time step to maximum possible
                dt = storm_dur - time
                #recalculate flow directions
                flowdirs, max_slopes = flow_router.calc_flowdirs(grid,z)
                #recalculate drainage area
                drain_area = accumulator.calc_flowacc(grid, z, flowdirs)
        
        return z       
        



# -*- coding: utf-8 -*-
"""
hex_lattice_tectonicizer.py

Models discrete normal-fault offset on a 2D hex lattice with a rectangular
shape and with one orientation of the nodes being vertical.

The intention here is to use a particle (LCA) model to represent the evolution
of a 2D hillslope, with the hex_lattice_tectonicizer serving to shift the nodes
either upward (simple vertical uplift relative to baselevel), or up and
sideways (representing motion on a fault plane).

Created on Mon Nov 17 08:01:49 2014

@author: gtucker
"""

from landlab import HexModelGrid
from numpy import amax, zeros, arange
from pylab import figure

_DEFAULT_NUM_ROWS = 20
_DEFAULT_NUM_COLS = 15
_TAN60 = 1.732


class HexLatticeTectonicizer(object):
    """Base class from which classes to represent particular baselevel/fault
    geometries are derived.
    """
    def __init__(self, grid=None, node_state=None):
        
        # If needed, create grid
        if grid is None:
            num_rows = _DEFAULT_NUM_ROWS
            num_cols = _DEFAULT_NUM_COLS
            self.grid = HexModelGrid(num_rows, num_cols, dx=1.0,
                                     orientation='vertical',
                                     shape='rect', reorient_links=True)
        else:
            # Make sure caller passed the right type of grid            
            assert (grid.orientation=='vertical'), \
                   'Grid must have vertical orientation'
                   
            # Keep a reference to the grid
            self.grid = grid
            
        # If needed, create node-state grid
        if node_state is None:
            self.node_state = self.grid.add_zeros('node', 'node_state_map')
        else:
            self.node_state = node_state
            
        # Remember the # of rows and cols
        self.nr = self.grid.number_of_node_rows
        self.nc = self.grid.number_of_node_columns
        

class LatticeNormalFault(HexLatticeTectonicizer):
    """Represents a 60 degree, left-dipping normal fault, and handles discrete
    offsets for a hex grid that has vertical columns and a rectangular shape.
    """
    def __init__(self, fault_x_intercept=0.0, grid=None, node_state=None):
        
        # Do the base class init
        super(LatticeNormalFault, self).__init__(grid, node_state)

        # Set up data structures:
        #   Make sure the footwall location is such that the fault actually
        #   cuts across the grid. This means the x intercept has to be, at
        #   the very least, no smaller than the biggest x-coordinate, and if
        #   there is an even number of columns, it must be smaller than that
        #   number minus 1/tangent 60 degrees (0.57735)
        assert (fault_x_intercept<amax(self.grid.node_x)-0.57735)
        
        #   Figure out which nodes are and are not within the footwall
        in_footwall = (self.grid.node_y<_TAN60*(self.grid.node_x-fault_x_intercept))
        
        #   Find the first of the bottom-row nodes that lies in the footwall
        self.first_fw_col = 0
        n = 0
        while not in_footwall[n]:
            n += self.nr
            self.first_fw_col += 1
            
        #   Remember the number of footwall rows in each column
        self.num_fw_rows = zeros(self.nc)
        for c in range(self.nc):
            current_row = 0
            while current_row<self.nr and in_footwall[current_row+c*self.nr]:
                self.num_fw_rows[c] += 1
                current_row += 1
            
        
    def do_offset(self, rock_state=1):
        """Applies 60-degree normal-fault offset to a hexagonal grid with
        vertical node orientation and rectangular arrangement of nodes.
        """        
        for c in range(self.grid.number_of_node_columns-1, self.first_fw_col-1, -1):
            row_offset = 2-c%2
            n_base_nodes = min(self.num_fw_rows[c], row_offset)
            bottom_node = c*self.grid.number_of_node_rows
            self.node_state[bottom_node:(bottom_node+n_base_nodes)] = rock_state
            indices = arange(n_base_nodes, self.num_fw_rows[c], dtype=int)+bottom_node
            if len(indices)>0:
                self.node_state[indices] = self.node_state[indices-(self.nr+row_offset)]
        
        if self.first_fw_col==0:
            self.node_state[:self.n_footwall_rows[0]] = rock_state


class LatticeUplifter(HexLatticeTectonicizer):
    """Handles vertical uplift of interior (not edges) for a hexagonal lattice
    with vertical node orientation and rectangular node arrangement.
    """
    def __init__(self, grid=None, node_state=None):
        
        # Do the base class init
        super(LatticeUplifter, self).__init__(grid, node_state)

        self.base_row_nodes = arange(self.nr, self.nr*(self.nc-1), self.nr)

    def uplift_interior_nodes(self, rock_state=1):
        
        for r in range(self.nr-1, 0, -1):
            self.node_state[self.base_row_nodes+r] = \
                    self.node_state[self.base_row_nodes+(r-1)]
        self.node_state[self.base_row_nodes] = rock_state

def main():

    lnf = LatticeNormalFault()

    for i in range(13):
        lnf.do_offset()
    lnf.grid.hexplot(lnf.node_state)
    
    lu = LatticeUplifter()
    lu.uplift_interior_nodes()
    figure(2)
    for i in range(5):
        lu.uplift_interior_nodes()
        lu.grid.hexplot(lu.node_state)
    
if __name__=='__main__':
    main()



#! /usr/env/python
"""
oriented_raster_lca.py: simple raster Landlab cellular automaton, with 
cell-pair transitions that depend on orientation (vertical or horizontal)

This file defines the OrientedRasterLCA class, which is a sub-class of 
LandlabCellularAutomaton that implements a simple, oriented, raster-grid
CA. Like its parent class, OrientedRasterLCA implements a continuous-time, 
stochastic, pair-based CA.

Created GT Sep 2014
"""

from numpy import zeros
from landlab_ca import LandlabCellularAutomaton, Transition
import landlab

_DEBUG = False

class OrientedRasterLCA(LandlabCellularAutomaton):
    
    def __init__(self, model_grid, node_state_dict, transition_list,
                 initial_node_states, track_cell_properties=False):
        
        if _DEBUG:
            print 'OrientedRasterLCA.__init__ here'

        # Make sure caller has sent the right grid type        
        assert (type(model_grid) is landlab.grid.raster.RasterModelGrid), \
               'model_grid must be a Landlab RasterModelGrid'
               
        # Define the number of distinct cell-pair orientations: here just 1,
        # because RasterLCA represents a non-oriented CA model.
        self.number_of_orientations = 2
        
        # Call the LandlabCellularAutomaton.__init__() method to do the rest of
        # the initialization
        super(OrientedRasterLCA, self).__init__(model_grid, node_state_dict, 
            transition_list, initial_node_states)
            
        if _DEBUG:
            print 'ORLCA:'
            print self.n_xn
            print self.xn_to
            print self.xn_rate
        

    def setup_array_of_orientation_codes(self):
        """
        Creates and configures an array that contain the orientation code for 
        each active link (and corresponding cell pair).
        
        Parameters
        ----------
        (none)
        
        Returns
        -------
        (none)
        
        Creates
        -------
        self.active_link_orientation : 1D numpy array
        
        Notes
        -----
        This overrides the method of the same name in landlab_ca.py.
        """
        # Create array for the orientation of each active link
        self.active_link_orientation = zeros(self.grid.number_of_active_links, dtype=int)
    
        # Set its value according to the different in y coordinate between each
        # link's TO and FROM nodes (the numpy "astype" method turns the
        # resulting array into integer format)
        dy = self.grid.node_y[self.grid.link_tonode[self.grid.active_links]] \
             - self.grid.node_y[self.grid.link_fromnode[self.grid.active_links]]
        self.active_link_orientation = dy.astype(int)
        #print 'i  from  to  dy'
        #for i in range(self.grid.number_of_active_links):
        #    print i, self.grid.activelink_fromnode[i], self.grid.activelink_tonode[i], dy[i], self.active_link_orientation[i]
        #print
        
        if _DEBUG:
            print self.active_link_orientation
            
            
if __name__=='__main__':
    mg = landlab.RasterModelGrid(3, 4, 1.0)
    nsd = {0 : 'yes', 1 : 'no'}
    xnlist = []
    xnlist.append( Transition( (0,1,0), (1,1,0), 1.0, 'frogging' ) )
    nsg = mg.add_zeros('node', 'node_state_grid')
    orlca = OrientedRasterLCA(mg, nsd, xnlist, nsg)
    print orlca.__dict__



#! /usr/env/python
"""

Python implementation of d8 routing scheme for
Landlab with a rectangular, uniform, mesh.

Some issues that might come up ...
This class relies on a method in model_grid called
find_node_in_direction_of_max_slope.  That method does not do
any boundary checking.  It needs to be done somewhere, maybe here,
maybe there.  Alternatively, if boundary elevations are always set
in a consistent way depending on the type of bounary, maybe no boundary
checking is needed?

Last updated NG 6/2013

"""

#from landlab.model_grid import RasterModelGrid
from numpy import *


class RouteFlowD8(object):
    """
    This class finds the steepest path among 8 possible directions, so
    diagonals are considered.
    The class assumes that the model is using a rectangular, uniform (raster)
    grid.
    """
    
    def __init__(self, num_cells):
        """
        This sets the num_cells parameter.
        This class assumes that the number of cells does not change after a 
        class item has been instantiated.
        """

        self.num_cells = num_cells
        self.initialize()
        
        #print 'RouteFlowD8.__init__'
        
    def initialize(self):
        """
        This sets up the flow direction vector.
        It is initialized to -1 for all nodes.
        A -1 flowdirs value indicates a boundary node.
        Is this method really needed?  Can we just put this in __init__?
        """
        
        self.flowdirs = -ones(self.num_cells, dtype=int)
      
    def calc_flowdirs(self, mg, z):
        """
        This assigns the flow directions using the function
        find_node_in_direction_of_max_slope in the model_grid.
        The flowdirs vector contains the node id that a node flows to. 
        If the node is a boundary node, the flowdirs vector has a value of -1.
        
        Method inputs: the model grid and elevation vector 
        Method returns: the flow direction vector
        """
        
        for i in range(0, self.num_cells):
            if mg.is_interior(i):
                self.flowdirs[i] = mg.find_node_in_direction_of_max_slope(z, i)
        
        return self.flowdirs        



#! /usr/bin/env python

"""
This component allows creation of mp4 animations of output from Landlab.
It does so by stitching together output from the conventional Landlab
static plotting routines from plot/imshow.py.

It is compatible with all Landlab grids, though cannot handle an evolving grid
as yet.

Initialize the video object vid at the start of your code, then simply call
vid.add_frame(grid, data) each time you want to add a frame. At the end of
the model run, call vid.produce_video().

CAUTION: This component may prove *very* memory-intensive. It is recommended
that the total number of frames included in the output multiplied by the
number of pixels (nodes) in the image not exceed XXXXXXXXX.

Due to some issues with codecs in matplotlib, at the moment on .gif output
movies are recommended. If this irritates you, you can modify your own 
PYTHONPATH to allow .mp4 compilation (try a google search for the warning raised
by this method for some hints). These (known) issues are apparently likely to 
resolve themselves in a future release of matplotlib.
"""

import numpy as np
import matplotlib.pyplot as plt
import matplotlib.animation as animation
from landlab.plot import imshow



class VideoPlotter(object):
    
    def __init__(self, grid, data_centering='node', start=None, stop=None, step=None):
        self.initialize(grid, data_centering, start, stop, step)
    
    def initialize(self, grid, data_centering, start, stop, step):
        """
        A copy of the grid is required.
        
        *data_centering* controls the type of data the video will be plotting.
        
        It can be set to:
            'node' (default)
            'active_node'
            'core_node'
            'cell'
            'active_cell'
        
        Start, stop, and step control when a frame is added. They are absolute
        times in the model run. All are optional.
        """
        options_for_data_centering = ['node',
                                      'active_node',
                                      'core_node',
                                      'cell',
                                      'active_cell']
        
        assert data_centering in options_for_data_centering, 'data_centering is not a valid type!'
        
        self.grid = grid
        #self.image_list = []
        self.data_list = []
        
        self.last_remainder = float('inf') #this controls the intervals at which to plot
        self.last_t = float('-inf')
        if start is None:
            start = float('-inf')
        if stop is None:
            stop = float('inf')
        self.step_control_tuple = (start,stop,step)
        
        #initialize the plots for the vid
        if data_centering=='node':
            self.centering = 'n'
            self.plotfunc = imshow.imshow_node_grid
        elif data_centering=='active_node':
            self.centering = 'n'
            self.plotfunc = imshow.imshow_active_node_grid
        elif data_centering=='core_node':
            self.centering = 'n'
            self.plotfunc = imshow.imshow_core_node_grid
        elif data_centering=='cell':
            self.centering = 'c'
            self.plotfunc = imshow.imshow_cell_grid
        else:
            self.centering = 'c'
            self.plotfunc = imshow.imshow_active_cell_grid
        
        self.randomized_name = "my_animation_"+str(int(np.random.random()*10000))
        self.fig = plt.figure(self.randomized_name) #randomized name 
        
    def add_frame(self, grid, data, elapsed_t, **kwds):
        """
        data can be either the data to plot (nnodes, or appropriately lengthed
        numpy array), or a string for grid field access.

        kwds can be any of the usual plotting keywords, e.g., cmap. 
        """
        if type(data)==str:
            if self.centering=='n':
                data_in = grid.at_node[data]
            elif self.centering=='c':
                data_in = grid.at_cell[data]
        else:
            data_in = data
            
        self.kwds = kwds
        
        if self.last_t<elapsed_t:
            try:
                normalized_elapsed_t = elapsed_t - self.start_t
            except AttributeError:
                self.start_t = elapsed_t
                normalized_elapsed_t = 0.
        else: #time has apparently gone "backwards"; reset the module
            #...note a *forward* jump in time wouldn't register
            self.clear_module()
            self.start_t = elapsed_t
            normalized_elapsed_t = 0.

        if self.step_control_tuple[0]<=elapsed_t<self.step_control_tuple[1]: #we're between start & stop
            if not self.step_control_tuple[2]: #no step provided
                print 'Adding frame to video at elapsed time ', elapsed_t
                self.data_list.append(data_in.copy())
            else:
                excess_fraction = normalized_elapsed_t%self.step_control_tuple[2]
                # Problems with rounding errors make this double check
                # necessary
                if excess_fraction < self.last_remainder or np.allclose(excess_fraction, self.step_control_tuple[2]):
                    print 'Adding frame to video at elapsed time ', elapsed_t
                    self.data_list.append(data_in.copy())
                self.last_remainder = excess_fraction
        self.last_t = elapsed_t
        
    
    def produce_video(self, interval=200, repeat_delay=2000, filename='video_output.gif', override_min_max=None):
        """
        Finalize and save the video of the data.
        
        interval and repeat_delay are the interval between frames and the repeat
            delay before restart, both in milliseconds.
        filename is the name of the file to save in the present working 
            directory. At present, only .gifs will implement reliably without
            tweaking Python's PATHs.
        override_min_max allows the user to set their own maximum and minimum
            for the scale on the plot. Use a len-2 tuple, (min, max).
        """
        print "Assembling video output, may take a while..."
        plt.figure(self.randomized_name)
        #find the limits for the plot:
        if not override_min_max:
            self.min_limit = np.amin(self.data_list[0])
            self.max_limit = np.amax(self.data_list[0])
            assert len(self.data_list) > 1, 'You must include at least two frames to make an animation!'
            for i in self.data_list[1:]: #assumes there is more than one frame in the loop
                self.min_limit = min((self.min_limit, np.amin(i)))
                self.max_limit = max((self.max_limit, np.amax(i)))
        else:
            self.min_limit=override_min_max[0]
            self.max_limit=override_min_max[1]
            
        self.fig.colorbar(self.plotfunc(self.grid, self.data_list[0],limits=(self.min_limit,self.max_limit),allow_colorbar=False, **self.kwds))
        ani = animation.FuncAnimation(self.fig, _make_image, frames=self._yield_image, interval=interval, blit=True, repeat_delay=repeat_delay)
        ani.save(filename, fps=1000./interval)
        plt.close()
        
        
    def _yield_image(self):
        """
        Helper function designed to generate image_list items for plotting,
        rather than storing them all.
        """
        
        for i in self.data_list:
            #yield self.grid.node_vector_to_raster(i)
            yield (i, self.plotfunc, (self.min_limit, self.max_limit), self.grid, self.kwds)

    
    def clear_module(self):
        """
        Wipe all internally held data that would cause trouble if module
        were to be rerun without being reinstantiated.
        """
        self.data_list = []


def _make_image(yielded_tuple):
    yielded_raster_data = yielded_tuple[0]
    plotfunc = yielded_tuple[1]
    limits_in = yielded_tuple[2]
    grid = yielded_tuple[3]
    kwds = yielded_tuple[4]
    im = plotfunc(grid, yielded_raster_data, limits=limits_in, allow_colorbar=False, **kwds)
    return im



# -*- coding: utf-8 -*-
""" generate_overland_flow.py 

 This component simulates overland flow using
 the 2-D numerical model of shallow-water flow
 over topography using the Bates et al. (2010)
 algorithm for storage-cell inundation modeling.

Written by Greg Tucker, Nicole Gasparini and Jordan Adams


"""
from landlab import Component, ModelParameterDictionary
from landlab.components.detachment_ltd_sed_trp.generate_detachment_ltd_transport import DetachmentLtdErosion
import pylab
import numpy as np
from matplotlib import pyplot as plt
from math import atan, degrees
import os
#from landlab.plot.video_out import VideoPlotter as vid
import csv

f =open('C:\Users\Jordan\Dropbox\z.csv','w')
_DEFAULT_INPUT_FILE = os.path.join(os.path.dirname(__file__),
                                  'overland_flow_input.txt')

class OverlandFlow(Component):
    
        
    
    '''  Landlab component that simulates overland flow using the Bates et al., (2010) approximations
of the 1D shallow water equations to be used for 2D flood inundation modeling. 
    
This component calculates discharge, depth and shear stress after some precipitation event across
any raster grid. Default input file is named "overland_flow_input.txt' and is contained in the
landlab.components.overland_flow folder.
        
    Inputs
    ------
    grid : Requires a RasterGridModel instance
        
    input_file : Contains necessary and optional inputs. If not given, default input file is used.
        - Manning's n is REQUIRED.
        - Storm duration is needed IF rainfall_duration is not passed in the initialization
        - Rainfall intensity is needed IF rainfall_intensity is not passed in the initialization
        - Model run time can be provided in initialization. If not it is set to the storm duration
        
    Constants
    ---------
    h_init : float
        Some initial depth in the channels. Default = 0.001 m
    g : float
        Gravitational acceleration, \frac{m}{s^2}
    alpha : float
        Non-dimensional time step factor from Bates et al., (2010)
    rho : integer
        Density of water, \frac{kg}{m^3}
    ten_thirds : float
        Precalculated value of \frac{10}{3} which is used in the implicit shallow water equation.
            
            
    >>> DEM_name = 'DEM_name.asc'
    >>> (rg, z) = read_esri_ascii(DEM_name) # doctest: +SKIP
    >>> of = OverlandFlow(rg) # doctest: +SKIP
       
'''
    _name = 'OverlandFlow'

    _input_var_names = set([
        'water_depth',
        'rainfall_intensity',
        'rainfall_duration',
        'elevation',
    ])
    _output_var_names = set([
        'water_depth',
        'water_discharge',
        'shear_stress',
        'water_discharge_at_nodes',
        'slope_at_nodes',
    ])

    _var_units = {
        'water_depth': 'm',
        'rainfall_intensity': 'm/s',
        'rainfall_duration': 's',
        'water_discharge': 'm3/s',
        'shear_stress': 'Pa',
        'water_discharge_at_nodes': 'm3/s',
        'slope_at_nodes': 'm/m',
        'elevation': 'm',
    }

    
    def __init__(self, grid, input_file=None, detach_ltd=True, **kwds):
        self.h_init = kwds.pop('h_init', 0.001)
        self.alpha = kwds.pop('alpha', 0.2)
        self.rho = kwds.pop('rho', 1000)
        self.m_n = kwds.pop('Mannings_n', 0.04)
        self.rainfall_intensity = kwds.pop('rainfall_intensity', None)
        self.rainfall_duration = kwds.pop('rainfall_duration', None)
        self.model_duration = kwds.pop('model_duration', None)       


        super(OverlandFlow, self).__init__(grid, **kwds)

        for name in self._input_var_names:
            if not name in self.grid.at_node:
                self.grid.add_zeros('node', name, units=self._var_units[name])

        for name in self._output_var_names:
            if not name in self.grid.at_node:
                self.grid.add_zeros('node', name, units=self._var_units[name])

        self._nodal_values = self.grid['node']   
        self._link_values = self.grid['active_link']

               
        self._water_discharge = grid.add_zeros('active_link', 'water_discharge', units=self._var_units['water_discharge'])                     
        self.g = 9.8               
        self.ten_thirds = 10./3.   
        self.current_time = 0.0
        self.m_n_sq = self.m_n*self.m_n 
        self.elapsed_time = 0
        self.q_at_node = []
        self.time = []
        
        self.detach_ltd = detach_ltd 

            
        # Create a ModelParameterDictionary instance for the inputs if they
        # are not provided as optional keyword arguments.
        
        MPD = ModelParameterDictionary()
        if input_file is None:
            input_file = _DEFAULT_INPUT_FILE
           
        
        MPD.read_from_file(input_file)    

        if self.rainfall_duration == None:
            self.rainfall_duration = MPD.read_float('RAINFALL_DURATION')
        if self.rainfall_intensity == None:
            self.rainfall_intensity = MPD.read_float('RAINFALL_INTENSITY')
        if self.model_duration == None:
            self.total_time = self.rainfall_duration
            
        if self.detach_ltd == True:
            self.dl = DetachmentLtdErosion(self.grid)
        
        
        # Set up grid arrays for state variables
        
        # Water Depth
        self.hstart = grid.zeros(centering='node') + self.h_init 
        self.h = grid.zeros(centering='node') + self.h_init     
        self.dhdt = grid.zeros(centering='cell') 
        self.dqds= grid.zeros(centering='node')

        # Discharge
        self.q = grid.zeros(centering='active_link') # unit discharge (m2/s)
        
        # Tau
        self.tau = grid.zeros(centering='node') # shear stress (Pascals)
        self.total_dzdt = grid.zeros(centering='node')
        self.h = self.hstart
        
    def flow_at_one_node(self, grid, z, study_node, **kwds):
        
        '''
        Outputs water depth, discharge and shear stress values through time at
        a user-selected point, defined as the "study_node" in function arguments.

        
        Inputs
        ------
        grid : Requires a RasterGridModel instance
        
        z : elevations drawn from the grid
        
        study_node : node ID for the the node at which discharge, depth and shear stress are calculated
        
        total_t : total model run time. If not provided as an argument or in the input_file, it is set to the storm_duration
        
        rainfall_intensity : rainfall intensity in m/s. If not provided as an argument, it must come from the input file.
        
        self.rainfall_duration : storm duration in seconds. If not provided as an argument, it must come from the input file.
        
        >>> study_row = 10
        >>> study_column = 10
        >>> rg.node_coords_to_id(study_row, study_column) # doctest: +SKIP
        >>> of.flow_at_one_node(rg, z, study_node, model_duration, storm_intensity, storm_duration) # doctest: +SKIP
        
        The study_node should NOT be a boundary node.
        '''

        self.q_node = grid.zeros(centering='node')

        self.interior_nodes = grid.get_active_cell_node_ids()
        self.active_links = grid.active_links

        self._water_depth = self._nodal_values['water_depth']
        self._slope_at_nodes = self._nodal_values['slope_at_nodes']

        self._shear_stress = self._nodal_values['shear_stress']
        self._water_discharge = self._link_values['water_discharge']
        
        self.rainfall_duration = kwds.pop('rainfall_duration', self.rainfall_duration)
        self.rainfall_intensity = kwds.pop('rainfall_intensity',self.rainfall_intensity) 
        self.interstorm_duration = kwds.pop('interstorm_duration', 0)
        
        self.model_duration = kwds.pop('model_duration', self.rainfall_duration)      
        self.z = z
        self.elapsed_time = kwds.pop('elapsed_time', 0)
        self.model_duration += self.interstorm_duration
        self.h_thresh = 1*(10**-6)
        
        self.study_node = study_node

         # Get a list of the interior cells and active links for our fields...
        self.interior_nodes = grid.get_active_cell_node_ids()
        self.active_links = grid.active_links
    
        # To track discharge at the study node through time, we create initially empty
        # lists for the node discharge, tau and depth. Time is also saved to help with
        # plotting later...

        ## Time array for x-axis        
        self.time = [0]
        
        # Discharge at study node
        self.q_study = [0]

        # Shear stress at study node
        self.tau_study = [0]
        
        # Water depth at study node
        self.depth_study = [0]
        
        # Initial value in each array is zero.

        # Some of the calculations (like discharge) take place at links. 
        # This next bit of code identifies the neighbor node of study_node
        # That is located in the direction of max slope.
        # Then, using that node ID and the ID of the new node, the study link
        # is identified.
    
        
        # New "NODE" arrays are created so that link values can be put onto
        # interior nodes in the grid.
        
        self.q_node = grid.zeros(centering='node')
        self.tau_node = grid.zeros(centering='node')
        self.dqds = grid.zeros(centering='node')
        
        # Main loop
        while self.elapsed_time < self.model_duration:
        
            # Calculate time-step size for this iteration (Bates et al., eq 14)
            dtmax = self.alpha*grid.dx/np.sqrt(self.g*np.amax(self.h)) 
            
            dt = min(dtmax, self.model_duration) 
        
            # Calculate the effective flow depth at active links. Bates et al. 2010
            # recommend using the difference between the highest water-surface
            # and the highest bed elevation between each pair of cells.
            
            zmax = grid.max_of_link_end_node_values(z) 
            w = self.h+self.z   
            wmax = grid.max_of_link_end_node_values(w) 
            hflow = wmax - zmax 
        
            # Calculate water-surface slopes across links
            water_surface_slope = grid.calculate_gradients_at_active_links(w)
       
            # Calculate the unit discharges (Bates et al., eq 11)
            above_thresh = np.where(hflow >self.h_thresh)
            below_thresh = np.where(hflow < self.h_thresh)
            
            self.q[above_thresh] = (self.q[above_thresh]-self.g*hflow[above_thresh]*dtmax*water_surface_slope[above_thresh])/ \
                (1.+self.g*hflow[above_thresh]*dtmax*0.06*0.06*(abs(self.q[above_thresh])/(hflow[above_thresh]**self.ten_thirds)))
                
            self.q[below_thresh] = 0.0
                    
            # Calculate water-flux divergence at nodes
            self.dqds = grid.calculate_flux_divergence_at_nodes(self.q)
        
            # Update rainfall rate
            if self.elapsed_time > self.rainfall_duration:
                self.rainfall_intensity = 0.
        
            # Calculate rate of change of water depth
            self.dhdt = self.rainfall_intensity-self.dqds
        
             #Second time-step limiter (experimental): make sure you don't allow
             #water-depth to go negative
            if np.amin(self.dhdt) < 0.:
                shallowing_locations = np.where(self.dhdt<0.)
                time_to_drain = -self.h[shallowing_locations]/self.dhdt[shallowing_locations]
                dtmax2 = self.alpha*np.amin(time_to_drain)
                dt = np.min([dtmax, dtmax2, self.model_duration])
            else:
                dt = dtmax
        
        
            # Update the water-depth field
            self.h[self.interior_nodes] = self.h[self.interior_nodes] + self.dhdt[self.interior_nodes]*dt

            # Get the water surface slope at across all nodes. 
            self.slopes_at_node, garbage = grid.calculate_steepest_descent_on_nodes(w, water_surface_slope)
            self._water_discharge_at_nodes = self.get_summed_out_discharge_at_nodes()

            # Calculate shear stress using the study node values
            tau_temp=self.rho*self.g*self.slopes_at_node[study_node]*self.h[study_node]
            
            # Add the shear stress value at study node to the shear stress array
            self.tau_study.append(tau_temp)
            
            # Add the water depth value at study node to the water depth array
            self.depth_study.append(self.h[study_node])
            
            # Add the discharge value at study node to the discharge array
            self.q_study.append(self._water_discharge_at_nodes[study_node])    

            # Append new model time to the time array.
            self.time.append(self.elapsed_time)
            
            # Update model run time and print elapsed time.
            self.elapsed_time += dt
            print "elapsed time", self.elapsed_time
            
        # Update our fields...
        self._water_depth[self.interior_nodes] = self.h[self.interior_nodes]
        self._water_discharge = self.q
        self._slope_at_nodes[self.interior_nodes] = self.slopes_at_node[self.interior_nodes]

          
    def update_at_one_point(self, grid, rainfall_duration, model_duration, rainfall_intensity, **kwds):
        '''
        The update_at_one_point() method, for now, requires our grid (which has fields associated with it),
        a new rainfall_duration, model_durationa and rainfall_intensity until I figure out some clever way to update using
        the MPD. 
        
        This function updates the overall elapsed time and passes it as an optional argument to flow_at_one_node().
        hstart is updated to be the depths from the field (_water_depth), model duration is updated to include past calls to
        flow_at_one_node() and/or update_at_one_node() depending on the past model run.
    
        Rainfall_duration is also updated so that model time will continue sequentially.
        '''
        
        # First, lets get the elapsed time from the last model run. 
        elapsed = self.elapsed_time
        
        # Set our initial depth condition to the final depth condition using fields.
        self.hstart = self._water_depth
        
        # Update the model duration and rain duration so that it begins where the last run left off.
        model_dur = model_duration + self.elapsed_time
        rain_dur = rainfall_duration+self.elapsed_time
        
        # Getting a new rainfall_intensity from our required arguments.
        intens= rainfall_intensity
        
        # Finally, call flow_at_one_node with our updated conditions.
        self.flow_at_one_node(grid, self.z, self.study_node, rainfall_duration=rain_dur, rainfall_intensity=intens, model_duration=model_dur, elapsed_time=elapsed)
        
        print '\n','\n'

            
                
    def flow_across_grid(self, grid, z, detach_ltd=True, **kwds):  
        '''
        Outputs water depth, discharge and shear stress values through time at
        every point in the input grid.

        
        Inputs
        ------
        grid : Requires a RasterGridModel instance
        
        z : elevations drawn from the grid
     
        total_t : total model run time. If not provided as an argument or in the input_file, it is set to the storm_duration
        
        rainfall_intensity : rainfall intensity in m/s. If not provided as an argument, it must come from the input file.
        
        rainduration : storm duration in seconds. If not provided as an argument, it must come from the input file.
        
        '''
        self.interior_nodes = grid.get_active_cell_node_ids()
        self.active_links = grid.active_links

        #self._water_depth = self._nodal_values['water_depth']
        self._shear_stress = self._nodal_values['shear_stress']
        self._water_discharge = self._link_values['water_discharge']
        self._slope_at_nodes = self._nodal_values['slope_at_nodes']
        self.elevation = self._nodal_values['elevation']

        self.rainfall_duration = kwds.pop('rainfall_duration', self.rainfall_duration)
        self.rainfall_intensity = kwds.pop('rainfall_intensity', self.rainfall_intensity)
        self.interstorm_duration = kwds.pop('interstorm_duration', 0)
        
        self.model_duration = kwds.pop('model_duration', self.rainfall_duration)      
        self._water_depth = self._nodal_values['water_depth']
        self.z = z
        self.elapsed_time = kwds.pop('elapsed_time', 0)
        self.model_duration += self.interstorm_duration
        self.h_thresh = 1*(10**-6)

        # And we create an array that keeps track of discharge changes through time.
        
        # Main loop...        
        
    
        while self.elapsed_time < self.model_duration:
            
            # Calculate time-step size for this iteration (Bates et al., eq 14)
            dtmax = self.alpha*grid.dx/np.sqrt(self.g*np.amax(self.h))     
        
            # Take the smaller of delt or calculated time-step
            dt = min(dtmax, self.model_duration)
        
            # Calculate the effective flow depth at active links. Bates et al. 2010
            # recommend using the difference between the highest water-surface
            # and the highest bed elevation between each pair of cells.
            
            zmax = grid.max_of_link_end_node_values(z) 
            w = self.h+self.z   
            wmax = grid.max_of_link_end_node_values(w) 
            hflow = wmax - zmax 

        
            # Calculate water-surface slopes across links
            water_surface_slope = grid.calculate_gradients_at_active_links(w)
       
            # Calculate the unit discharges (Bates et al., eq 11)
            above_thresh = np.where(hflow >self.h_thresh)
            below_thresh = np.where(hflow < self.h_thresh)
            
            self.q[above_thresh] = (self.q[above_thresh]-self.g*hflow[above_thresh]*dtmax*water_surface_slope[above_thresh])/ \
                (1.+self.g*hflow[above_thresh]*dtmax*0.06*0.06*(abs(self.q[above_thresh])/(hflow[above_thresh]**self.ten_thirds)))
                
            self.q[below_thresh] = 0.0

                    
            # Calculate water-flux divergence at nodes
            self.dqds = grid.calculate_flux_divergence_at_nodes(self.q,self.dqds)
            
            # Update rainfall rate
            if self.elapsed_time > self.rainfall_duration:
                self.rainfall_intensity = 0
            
            # Calculate rate of change of water depth
            self.dhdt = self.rainfall_intensity-self.dqds
            
            # Second time-step limiter (experimental): make sure you don't allow
            # water-depth to go negative
            if np.amin(self.dhdt) < 0.:
                shallowing_locations = np.where(self.dhdt<0.)
                time_to_drain = -self.h[shallowing_locations]/self.dhdt[shallowing_locations]
                dtmax2 = self.alpha*np.amin(time_to_drain)
                dt = np.min([dtmax, dtmax2, self.model_duration])
            else:
                dt = dtmax
        
            # Update the water-depth field
            self.h[self.interior_nodes] = self.h[self.interior_nodes] + self.dhdt[self.interior_nodes]*dt

            # Now we can calculate shear stress across the grid...

            # Get water surface elevation at each interior node.
            w = self.h+self.z   
        
            # Using water surface elevation, calculate water surface slope at each interior node.
            self.slopes_at_node, dwnstr_nodes = grid.calculate_steepest_descent_on_nodes(w, water_surface_slope)
            self.tau = self.rho*self.g*self.slopes_at_node*self.h

            # Let's call detachment ltd erosion..
            self._water_discharge_at_nodes = self.get_summed_out_discharge_at_nodes()
            self.slopes_at_node[np.where(self.slopes_at_node<0)]=0.0
            if self.detach_ltd==True:
                new_z, dzdt = self.dl.change_elev(self.z, self.slopes_at_node,self._water_discharge_at_nodes)
            self.z_f = new_z
                
            # Update model run time.
            self.total_dzdt += dzdt
            self.elapsed_time += dt
            print "elapsed time", self.elapsed_time
        
        # Update our fields after the loop...
        self._water_depth[self.interior_nodes]= self.h.take(self.interior_nodes)
        self._shear_stress[self.interior_nodes] = self.tau.take(self.interior_nodes)
        self._water_discharge = self.q
        self._slope_at_nodes[self.interior_nodes] = self.slopes_at_node.take(self.interior_nodes)



    def update_across_grid(self, grid, rainfall_duration, model_duration, rainfall_intensity, **kwds):
        '''
        The update_across_grid() method, for now, requires our grid (which has fields associated with it),
        a new rainfall_duration, model_durationa and rainfall_intensity until I figure out some clever way to update using
        the MPD. 
        
        This function updates the overall elapsed time and passes it as an optional argument to flow_across_grid().
        hstart is updated to be the depths from the field (_water_depth), model duration is updated to include past calls to
        flow_across_grid() and/or update_across_grid() depending on the past model run.
    
        Rainfall_duration is also updated so that model time will continue sequentially.
        '''  
        # First, lets get the elapsed time from the last model run. 
        elapsed = self.elapsed_time
        
        # Set our initial depth condition to the final depth condition using fields.
        self.hstart = self._water_depth
        
        # Update the model duration and rain duration so that it begins where the last run left off.
        model_dur = model_duration + self.elapsed_time
        rain_dur = rainfall_duration+self.elapsed_time
        
        # Getting a new rainfall_intensity from our required arguments.
        intens= rainfall_intensity

        # Finally, call flow_across_grid() using our updated conditions
        self.flow_across_grid(grid, self.z, rainfall_duration=rain_dur, rainfall_intensity=intens, model_duration=model_dur, elapsed_time=elapsed)

    
        
    def get_summed_out_discharge_at_nodes(self):
        ''' 
        For each interior node, this method finds the links where discharge is flowing out and maps
        the sum of these "q_out" links to the respective interior node
        '''
        discharge_array = self.q[self.grid.node_activelinks(self.interior_nodes)]

        south = discharge_array[0]
        west = discharge_array[1]
        north = discharge_array[2]
        east = discharge_array[3]  
      
        south[np.where(south>0)] = 0.0
        west[np.where(west>0)] = 0.0
        
        south[np.where(south<0)] = south[np.where(south<0)]*-1
        west[np.where(west<0)] = west[np.where(west<0)]*-1
        

        north[np.where(north<0)] =0.0
        east[np.where(east<0)] =0.0

        q_node = south+west+north+east 
        self.q_interior_nodes = self.grid.zeros(centering='node')
        self.q_interior_nodes[self.interior_nodes] = q_node
        self._water_discharge_at_nodes = self.q_interior_nodes
        return self._water_discharge_at_nodes
        

    
    def plot_at_one_node(self):
        
        '''This method must follow a call to the flow_at_one_node() method.
        
        It plots depth, discharge and shear stress through time at the study
        node that was called in the flow_at_one_node() method.
        
           Three windows will appear after this method is called:
               1. Discharge through time is plotted as a solid blue line.
               2. Shear stress through time is plotted as a solid red line.
               3. Water depth through time is plotted as a solid cyan line.
        '''       
        plt.figure('Discharge at Study Node')
        plt.plot(self.time, self.q_study, 'b-')
        plt.legend(loc=1)
        plt.ylabel('Discharge, m^3/s')
        plt.xlabel('Time, s')
  
        plt.figure('Shear Stress at Study Node')
        plt.plot(self.time, self.tau_study, 'r-')
        plt.ylabel('Shear Stress, Pa')
        plt.xlabel('Time, s')
        plt.legend(loc=1)
        
        plt.figure('Water Depth at Study Node')
        plt.plot(self.time, self.depth_study, 'c-')
        plt.ylabel('Water Depth, m')
        plt.xlabel('Time, s')
        plt.legend(loc=1)
        plt.show()
        



from landlab.components.flow_routing.route_flow_dn import FlowRouter
from landlab.components.stream_power.fastscape_stream_power import SPEroder
from landlab import ModelParameterDictionary
from landlab.plot import imshow
from landlab.plot.video_out import VideoPlotter

from landlab import RasterModelGrid
import numpy as np
import pylab

from time import time

#get the needed properties to build the grid:
input_file = './stream_power_params.txt'
inputs = ModelParameterDictionary(input_file)
nrows = inputs.read_int('nrows')
ncols = inputs.read_int('ncols')
dx = inputs.read_float('dx')
leftmost_elev = inputs.read_float('leftmost_elevation')
initial_slope = inputs.read_float('initial_slope')
uplift_rate = inputs.read_float('uplift_rate')

runtime = 40.
dt = 1.

nt = int(runtime//dt)
uplift_per_step = uplift_rate * dt

#instantiate the grid object
mg = RasterModelGrid(nrows, ncols, dx)
#set up its boundary conditions (bottom, right, top, left is inactive)
mg.set_inactive_boundaries(False, True, False, True)

##create the elevation field in the grid:
#create the field
mg.create_node_array_zeros('topographic_elevation')
z = mg.create_node_array_zeros() + leftmost_elev
z += initial_slope*np.amax(mg.node_y) - initial_slope*mg.node_y
#put these values plus roughness into that field
mg['node'][ 'topographic_elevation'] = z + np.random.rand(len(z))/100000.

# Display a message
print 'Running ...' 

#instantiate the components:
fr = FlowRouter(mg)
sp = SPEroder(mg, input_file)
vid = VideoPlotter(mg, data_centering='node')

time_on = time()
#perform the loops:
for i in xrange(nt):
    print 'loop ', i
    mg['node']['topographic_elevation'][mg.core_nodes] += uplift_per_step
    mg = fr.route_flow(grid=mg)
    mg = sp.erode(mg)
    #vid.add_frame(mg, 'topographic_elevation')
    vid.add_frame(mg, mg.hillshade(alt=15.), cmap='gray')
    
 
print 'Completed the simulation. Plotting...'

time_off = time()

#Finalize and plot

elev = mg['node']['topographic_elevation']
#imshow.imshow_node_grid(mg, elev)

print('Done.')
print 'Time: ', time_off-time_on

#pylab.show()

vid.produce_video(override_min_max=(0,1))
#vid.produce_video()



import numpy as np
from pylab import * 
from landlab import RasterModelGrid
from landlab.plot.imshow import imshow_grid
from landlab.components.dem_support.dem_boundary_conditions import WatershedBoundaryConditions
from random import uniform
from landlab.components.simple_power_law_incision.power_law_fluvial_eroder import PowerLawIncision
from landlab.components.flow_routing.flow_routing_D8 import RouteFlowD8
from landlab.components.flow_accum.flow_accumulation2 import AccumFlow
import matplotlib.pyplot as plt
#reload(flow_routing_D8)
#reload(flow_accumulation)
#reload(raster)

def main():
    nr = 5
    nc = 6
    nnodes = nr*nc
    dx=1
    #instantiate grid
    rg = RasterModelGrid(nr, nc, dx)
    #rg.set_inactive_boundaries(False, False, True, True)
    
    nodata_val=-1
    z  = nodata_val*np.ones( nnodes )    
    #set-up interior elevations with random numbers
    #for i in range(0, nnodes):
    #    if rg.is_interior(i):
    #        elevations[i]=random.random_sample()
    
    #set-up elevations
    helper = [7,8,9,10,13,14,15,16]
    for i in xrange(0, len(helper)):
        #print 'helper[i]', helper[i]
        z[helper[i]]=2+uniform(-0.5,0.5)
    helper = [19,20,21,22]
    for i in xrange(0, len(helper)):
        z[helper[i]]=3+uniform(-0.5,0.5)
        
    z[7]=1
    
    #set up boundary conditions
    bc=WatershedBoundaryConditions()
    outlet_loc = bc.set_bc_find_outlet(rg, z, nodata_val)
    zoutlet=z[outlet_loc]
    
    #some helper and parameter values
    total_run_time = 500000 #yr
    uplift_rate = 0.001 #m/yr
    rain_rate = 1 #m/yr
    storm_duration = 50 #years
    
    elapsed_time = 0 #years
    
    #set up fluvial incision component
    incisor = PowerLawIncision('input_file.txt',rg)
    
    #print "elevations before ", rg.node_vector_to_raster(z)
    #interior_nodes are the nodes on which you will be operating
    interior_nodes = rg.get_active_cell_node_ids()
    
    while elapsed_time < total_run_time:
        #uplift the landscape
        z[interior_nodes] = z[interior_nodes]+uplift_rate * storm_duration
        z[outlet_loc]=zoutlet
        #erode the landscape
        z = incisor.run_one_storm(rg,z,rain_rate,storm_duration)
        #update the time
        elapsed_time = elapsed_time+storm_duration
    
    #below purely for plotting reasons
    #instantiate variable of type RouteFlowD8 Class
    flow_router = RouteFlowD8(len(z))
    #initial flow direction
    flowdirs, max_slopes = flow_router.calc_flowdirs(rg,z)
    #insantiate variable of type AccumFlow Class
    accumulator = AccumFlow(rg)
    #initial flow accumulation
    drain_area = accumulator.calc_flowacc(rg, z, flowdirs)
    
    #m,b = polyfit(log10(drain_area[interior_nodes]), log10(max_slopes[interior_nodes]), 1) 
    z[interior_nodes] = z[interior_nodes]+uplift_rate * storm_duration
    z[outlet_loc]=zoutlet
    
    plt.loglog(np.array(drain_area),np.array(max_slopes),'ro',)
    plt.show()
    imshow_grid(rg,z, values_at='node')
    #print "elevations after", rg.node_vector_to_raster(z)
    #print "drainage area after", rg.node_vector_to_raster(drain_area)
    #print "slope after", rg.node_vector_to_raster(max_slopes)
    #print "flowdirs after", rg.node_vector_to_raster(flowdirs)
    
    
if __name__ == '__main__':
    main()




#################################################################
##
##  Cellular Automaton component design for Vegetation establishment/mortality
##
##  Sai Nudurupati and Erkan Istanbulluoglu - 26 Nov 2014
#################################################################

from landlab import Component
import numpy as np

_VALID_METHODS = set(['Grid'])
GRASS = 0
SHRUB = 1
TREE = 2
BARE = 3
SHRUBSEEDLING = 4
TREESEEDLING = 5

def assert_method_is_valid(method):
    if method not in _VALID_METHODS:
        raise ValueError('%s: Invalid method name' % method)


class VegCA(Component):
    """
    Landlab component that implements 1D and 2D vegetation dynamics
    model.

    >>> from landlab import RasterModelGrid
    >>> from landlab.components.vegetation_ca.CA_Veg import VegCA
    >>> grid = RasterModelGrid(5, 4, 1.e4)
    >>> veg_ca = VegCA(grid)
    >>> veg_ca.name
    'VegCA'
    """
    _name = 'VegCA'

    _input_var_names = set([
        'CumulativeWaterStress',
        'VegetationType',
    ])

    _output_var_names = set([
        'PlantLiveIndex',
        'PlantAge',
    ])

    _var_units = {
        'CumulativeWaterStress' : 'Pa',
        'VegetationType'  : 'None',
        'PlantLiveIndex'  : 'Pa',
        'PlantAge'        : 'Years',
    }

    def __init__(self, grid, **kwds):
        self._method = kwds.pop('method', 'Grid')
        self._Pemaxg = kwds.pop('Pemaxg', 0.35)   # Pe-max-grass - max probability
        self._Pemaxsh = kwds.pop('Pemaxsh', 0.2)    # Pe-max-shrub
        self._Pemaxtr = kwds.pop('Pemaxtr', 0.25)   # Pe-max-tree
        self._INg = kwds.pop('ING', 2)  # Allelopathic effect on grass from creosotebush
        self._th_g = kwds.pop('ThetaGrass', 0.62)  # grass
        self._th_sh = kwds.pop('ThetaShrub', 0.8) # shrub - Creosote
        self._th_tr = kwds.pop('ThetaTree', 0.72) # Juniper pine
        self._th_sh_s = kwds.pop('ThetaShrubSeedling', 0.64) # shrub seedling
        self._th_tr_s = kwds.pop('ThetaTreeSeedling', 0.57) # Juniper pine seedling
        self._Pmb_g = kwds.pop('PmbGrass', 0.05) # Background mortality probability - grass
        self._Pmb_sh = kwds.pop('PmbShrub', 0.01) # shrub
        self._Pmb_tr = kwds.pop('PmbTree', 0.01) # tree
        self._Pmb_sh_s = kwds.pop('PmbShrubSeedling', 0.03) # shrub seedling
        self._Pmb_tr_s = kwds.pop('PmbTreeSeedling', 0.03) # tree seedling
        self._tpmax_sh = kwds.pop('tpmaxShrub', 600) # Maximum age - shrub
        self._tpmax_tr = kwds.pop('tpmaxTree', 350) # Maximum age - tree
        self._tpmax_sh_s = kwds.pop('tpmaxShrubSeedling', 18) # Maximum age - shrub seedling
        self._tpmax_tr_s = kwds.pop('tpmaxTreeSeedling', 20) # Maximum age - tree seedling


        assert_method_is_valid(self._method)

        super(VegCA, self).__init__(grid, **kwds)

        for name in self._input_var_names:
            if name not in self.grid.at_cell:
                self.grid.add_zeros('cell', name, units=self._var_units[name])

        for name in self._output_var_names:
            if name not in self.grid.at_cell:
                self.grid.add_zeros('cell', name, units=self._var_units[name])

        self._cell_values = self.grid['cell']

        if (np.where(grid['cell']['VegetationType'] != 0)[0].shape[0] == 0):
            grid['cell']['VegetationType'] =                        \
                                    np.random.randint(0,6,grid.number_of_cells)

        VegType = grid['cell']['VegetationType']
        tp = np.zeros(grid.number_of_cells, dtype = int)
        tp[VegType == TREE] = np.random.randint(0,self._tpmax_tr,
                                    np.where(VegType==TREE)[0].shape)
        tp[VegType == SHRUB] = np.random.randint(0,self._tpmax_sh,
                                    np.where(VegType==SHRUB)[0].shape)
        tp[VegType == TREESEEDLING] = np.random.randint(0,self._tpmax_tr_s,
                                    np.where(VegType==TREESEEDLING)[0].shape)
        tp[VegType == SHRUBSEEDLING] = np.random.randint(0,self._tpmax_sh_s,
                                    np.where(VegType==SHRUBSEEDLING)[0].shape)
        grid['cell']['PlantAge'] = tp


    def update(self, Edit_VegCov = True, time_elapsed = 1):

        self._VegType = self._cell_values['VegetationType']
        self._CumWS   = self._cell_values['CumulativeWaterStress']
        self._live_index = self._cell_values['PlantLiveIndex']
        self._tp = self._cell_values['PlantAge'] + time_elapsed

        # Check if shrub and tree seedlings have matured
        shrub_seedlings = np.where(self._VegType == SHRUBSEEDLING)[0]
        tree_seedlings = np.where(self._VegType == TREESEEDLING)[0]
        matured_shrubs = np.where(self._tp[shrub_seedlings] >
                                    self._tpmax_sh_s)[0]
        matured_trees = np.where(self._tp[tree_seedlings] >
                                    self._tpmax_tr_s)[0]
        self._VegType[shrub_seedlings[matured_shrubs]] = SHRUB
        self._VegType[tree_seedlings[matured_trees]] = TREE
        self._tp[shrub_seedlings[matured_shrubs]] = 0
        self._tp[tree_seedlings[matured_trees]] = 0

        # Establishment
        self._live_index = 1 - self._CumWS      # Plant live index = 1 - WS
        bare_cells = np.where(self._VegType == BARE)[0]
        n_bare = len(bare_cells)
        first_ring = self.grid.get_looped_cell_neighbor_list(bare_cells)
        second_ring =                                                   \
            self.grid.get_second_ring_looped_cell_neighbor_list(bare_cells)
        veg_type_fr = self._VegType[first_ring]
        veg_type_sr = self._VegType[second_ring]
        Sh_WS_fr = WS_PFT( veg_type_fr, SHRUB, self._live_index[first_ring] )
        Tr_WS_fr = WS_PFT( veg_type_fr, TREE, self._live_index[first_ring] )
        Tr_WS_sr = WS_PFT( veg_type_sr, TREE, self._live_index[second_ring] )

        n = count(veg_type_fr, SHRUB)
        Phi_sh = Sh_WS_fr/8.
        Phi_tr = (Tr_WS_fr + Tr_WS_sr/2.)/8.
        Phi_g = np.mean(self._live_index[np.where(self._VegType == GRASS)])
        Pemaxg = self._Pemaxg * np.ones(n_bare)
        Pemaxsh = self._Pemaxsh * np.ones(n_bare)
        Pemaxtr = self._Pemaxtr * np.ones(n_bare)
        Peg = np.amin(np.vstack((Phi_g/(n*self._INg),Pemaxg)),axis = 0)
        Pesh = np.amin(np.vstack((Phi_sh, Pemaxsh)), axis = 0)
        Petr = np.amin(np.vstack((Phi_tr, Pemaxtr)), axis = 0)
        Select_PFT_E = np.random.choice([GRASS, SHRUBSEEDLING, TREESEEDLING],
                                        n_bare)
        # Grass - 0; Shrub Seedling - 4; Tree Seedling - 5
        Pest = np.choose(Select_PFT_E, [Peg, 0, 0, 0, Pesh, Petr])
        # Probability of establishment
        R_Est = np.random.rand(n_bare)
        # Random number for comparison to establish
        Establish = np.int32(np.where(np.greater_equal(Pest, R_Est)==True)[0])
        self._VegType[bare_cells[Establish]] = Select_PFT_E[Establish]
        self._tp[bare_cells[Establish]] = 0

        # Mortality
        plant_cells = np.where(self._VegType != BARE)[0]
        n_plant = len(plant_cells)
        Theta = np.choose(self._VegType[plant_cells],
                    [ self._th_g, self._th_sh, self._th_tr,
                            0, self._th_sh_s, self._th_tr_s])
        PMd = self._CumWS[plant_cells] - Theta
        PMd[PMd < 0.] = 0.
        tpmax = np.choose(self._VegType[plant_cells],
                    [ 200000, self._tpmax_sh, self._tpmax_tr,
                            0, self._tpmax_sh_s, self._tpmax_tr_s])
        PMa = np.zeros(n_plant)
        tp_plant = self._tp[plant_cells]
        tp_greater = np.where(tp_plant>0.5*tpmax)[0]
        PMa[tp_greater] = ((tp_plant[tp_greater] - 0.5 * tpmax[tp_greater]) /
                           (0.5 * tpmax[tp_greater])) - 1
        PMb = np.choose( self._VegType[plant_cells],
                            [self._Pmb_g, self._Pmb_sh, self._Pmb_tr, 0,
                                self._Pmb_sh_s, self._Pmb_tr_s] )
        PM = PMd + PMa + PMb
        PM[PM>1.] = 1.
        R_Mor = np.random.rand(n_plant) # Random number for comparison to kill
        Mortality = np.int32(np.where(np.greater_equal(PM, R_Mor) == True)[0])
        self._VegType[plant_cells[Mortality]] = BARE
        self._tp[plant_cells[Mortality]] = 0

        self._cell_values['PlantAge'] = self._tp

        if Edit_VegCov:
            self.grid['cell']['VegetationCover'] =             \
                                    np.zeros(self.grid.number_of_cells)
            self.grid['cell']['VegetationCover'][self._VegType != BARE] = 1.

        # For debugging purposes
        self._bare_cells = bare_cells
        self._Established = bare_cells[Establish]
        self._plant_cells = plant_cells
        self._Mortified = plant_cells[Mortality]



def count( Arr, value ):
    Res = np.zeros(Arr.shape[0],dtype = int)
    x,y = Arr.shape
    for i in range(0,x):
        for j in range(0,y):
            if Arr[i][j] == value:
                Res[i] += 1
    return Res

def WS_PFT( VegType, PlantType, WS ):
    Phi = np.zeros(WS.shape[0])
    x,y = WS.shape
    for i in range(0,x):
        for j in range(0,y):
            if VegType[i][j] == PlantType:
                Phi[i] += WS[i][j]
    return Phi
